class User < ApplicationRecord
  attr_accessor :password

  has_many :topics
  has_many :comments
  has_many :attachments

  before_save :encrypt_password

  validates :name, presence: true
  validates :email, { presence: true, uniqueness: true }
  validates :password, confirmation: true, if: :password_required?

  def self.encrypt(pass, salt)
    Digest::SHA1.hexdigest pass+salt
  end

  def self.authenticate(email, password)
    user = User.where(email: email).take
    user && user.authenticated?(password) ? user : nil
  end

  def encrypt_password
    return if password.blank?
    if self.new_record?
      self.salt = SecureRandom.base64(16)
    end
    self.encrypted_password = User.encrypt(password, salt)
  end

  def authenticated?(password)
    self.encrypted_password == User.encrypt(password, self.salt)
  end

  def password_required?
    self.new_record? || !self.password.blank?
  end

  def to_s
    self.name
  end
end
